﻿using System;
using System.Net;
using System.Net.Sockets;

namespace ELTE.Net.ChatRoom.Tcp.ClientInViewModel.Model
{
    /// <summary>
    /// Hálózati kommunikációt felügyelő típus.
    /// </summary>
    public class NetworkManager
    {
        /// <summary>
        /// Kapcsolatváltás eseménye.
        /// </summary>
        public event EventHandler ConnectionChanged;
        /// <summary>
        /// Sikertelen végrehajtás eseménye.
        /// </summary>
        public event EventHandler OperationFailed;
        /// <summary>
        /// Üzenetfogadás eseménye.
        /// </summary>
        public event EventHandler MessageReceived;

        private Socket _ChatSocket;

        /// <summary>
        /// Aktuálisan fogadott üzenet lekérdezése.
        /// </summary>
        public String CurrentMessage { get; private set; }
        /// <summary>
        /// Kapcsolati állapot lekérdezése.
        /// </summary>
        public Boolean Connected { get; private set; }

        /// <summary>
        /// Csatlakozás távoli végponthoz.
        /// </summary>
        /// <param name="address">A távoli gép címe.</param>
        /// <param name="port">A távoli gép portszáma.</param>
        public void Connect(String address, Int32 port)
        {
            if (_ChatSocket != null && _ChatSocket.Connected) // socket bezárása, ha még nyitva van
            {
                _ChatSocket.Shutdown(SocketShutdown.Both); // mindkét oldali leállítás
                System.Threading.Thread.Sleep(10);
                _ChatSocket.Close();

                Connected = false;
                if (ConnectionChanged != null)
                    ConnectionChanged(this, EventArgs.Empty);
            }
            _ChatSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            // TCP socket létrehozása
            IPEndPoint serverEndPoint = new IPEndPoint(IPAddress.Parse(address), port);
            // táboli végpont
            _ChatSocket.Blocking = false; // nem blokkoló mód

            SocketAsyncEventArgs eArgs = new SocketAsyncEventArgs(); // eseményargumentum létrehozása
            eArgs.RemoteEndPoint = serverEndPoint; // távoli végpont
            eArgs.Completed += new EventHandler<SocketAsyncEventArgs>(Connect_Completed); // befejezéskor futtatandó eseménykezelő

            _ChatSocket.ConnectAsync(eArgs);
        }

        /// <summary>
        /// Üzenetküldés.
        /// </summary>
        /// <param name="message">A továbbítandó özenet.</param>
        public void Send(String message)
        {
            SocketAsyncEventArgs eArgs = new SocketAsyncEventArgs();
            eArgs.RemoteEndPoint = _ChatSocket.RemoteEndPoint;
            Byte[] buffer = System.Text.Encoding.UTF8.GetBytes(message); // átalakítás byte-tömbbé
            eArgs.SetBuffer(buffer, 0, buffer.Length); // küldendő üzenet beállítása

            _ChatSocket.SendAsync(eArgs);
        }

        private void Connect_Completed(object sender, SocketAsyncEventArgs e)
        {
            if (e.SocketError == SocketError.Success) // ha sikeres volt a művelet
            {
                Connected = true;
                if (ConnectionChanged != null)
                    ConnectionChanged(this, EventArgs.Empty); // jelezzük, hogy változott a kapcsolat állapota

                e.Dispose(); // régi eseményargumentum megsemmisítése

                SocketAsyncEventArgs eArgs = new SocketAsyncEventArgs();
                eArgs.RemoteEndPoint = e.RemoteEndPoint;
                eArgs.SetBuffer(new Byte[1024], 0, 1024); // üres tömb létrehozása az üzenetfogadáshoz
                eArgs.Completed += new EventHandler<SocketAsyncEventArgs>(Receive_Completed);

                _ChatSocket.ReceiveAsync(eArgs);
            }
            else
            {
                OperationFailed(this, EventArgs.Empty);
            }
        }

        private void Receive_Completed(object sender, SocketAsyncEventArgs e)
        {
            if (e.SocketError == SocketError.Success)
            {
                CurrentMessage = System.Text.Encoding.UTF8.GetString(e.Buffer, 0, e.BytesTransferred);

                if (MessageReceived != null)
                    MessageReceived(this, EventArgs.Empty);

                e.Dispose();

                SocketAsyncEventArgs eArgs = new SocketAsyncEventArgs();
                eArgs.RemoteEndPoint = e.RemoteEndPoint;
                eArgs.SetBuffer(new Byte[1024], 0, 1024);
                eArgs.Completed += new EventHandler<SocketAsyncEventArgs>(Receive_Completed);

                _ChatSocket.ReceiveAsync(eArgs);
            }
            else
            {
                if (_ChatSocket != null && _ChatSocket.Connected) // socket bezárása, ha még nyitva van
                {
                    _ChatSocket.Shutdown(SocketShutdown.Both); // mindkét oldali leállítás
                    System.Threading.Thread.Sleep(10);
                    _ChatSocket.Close();
                }
                Connected = false;
                if (OperationFailed != null)
                    OperationFailed(this, EventArgs.Empty);
            }
        }

        public void Close()
        {
            if (_ChatSocket != null && _ChatSocket.Connected) // bezárás előtt le kell kapcsolni a csatlakozást
            {
                _ChatSocket.Shutdown(SocketShutdown.Both); // kapcsolat leállítása
                _ChatSocket.Close();

                Connected = false;
                if (ConnectionChanged != null)
                    ConnectionChanged(this, EventArgs.Empty);
            }
        }

    }
}
