﻿using System;
using System.Net;
using System.Net.Sockets;
using ELTE.Net.GameServer.Shared;

namespace ELTE.Net.BallGame.Client.Model
{
    /// <summary>
    /// Játékok eseményargumentum típusa.
    /// </summary>
    public class GamesEventArgs : EventArgs
    {
        /// <summary>
        /// Játékok lekérdezése, vagy beállítása.
        /// </summary>
        public Game[] Games { get; set; }
    }

    /// <summary>
    /// Jtáékállapot eseményargumentum típusa.
    /// </summary>
    public class GameStateEventArgs : EventArgs
    {
        /// <summary>
        /// Ütőpozíció lekérdezése, vagy beállítása.
        /// </summary>
        public Int32 PadPosition { get; set; }
    }

    /// <summary>
    /// Hálózatkezelő típusa.
    /// </summary>
    public class NetworkManager
    {
        private Socket _Socket; // hálózati csatoló
        private Player _Player; // játékos
        private Game _Game; // aktuális játék

        public Boolean Connected { get { return _Socket != null && _Socket.Connected; } }

        /// <summary>
        /// Kapcsolatváltozás eseménye.
        /// </summary>
        public event EventHandler ConnectionChanged;
        /// <summary>
        /// Játéklista fogadás eseménye.
        /// </summary>
        public event EventHandler<GamesEventArgs> GamesReceived;
        /// <summary>
        /// Játékbeli lépés eseménye.
        /// </summary>
        public event EventHandler<GameStateEventArgs> GameStepReceived;
        /// <summary>
        /// Játékindítás eseménye.
        /// </summary>
        public event EventHandler GameStarted;
        /// <summary>
        /// Játék vége.
        /// </summary>
        public event EventHandler GameOver;

        /// <summary>
        /// Lecsatlakozás.
        /// </summary>
        public void Disconnect()
        {
            _Socket.Close();
            if (ConnectionChanged != null)
                ConnectionChanged(this, EventArgs.Empty);
        }
        /// <summary>
        /// Csatlakozás.
        /// </summary>
        /// <param name="address">A szerver címe.</param>
        /// <param name="playerName">A játékos neve.</param>
        public void Connect(IPAddress address, String playerName)
        {
            _Socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            _Player = new Player { PlayerName = playerName };

            SocketAsyncEventArgs eArgs = new SocketAsyncEventArgs();
            eArgs.RemoteEndPoint = new IPEndPoint(address, 4503);
            eArgs.Completed += new EventHandler<SocketAsyncEventArgs>(Connection_Completed);
            _Socket.ConnectAsync(eArgs);
        }

        private void Connection_Completed(object sender, SocketAsyncEventArgs e)
        {
            if (e.SocketError == SocketError.Success) // ha sikeres volt a művelet
            {
                if (ConnectionChanged != null)
                    ConnectionChanged(this, EventArgs.Empty); // jelezzük a kapcsolatváltást

                SendMessage(MessageCode.Login, _Player); // bejelentkező üzenet

                SocketAsyncEventArgs eArgs = new SocketAsyncEventArgs(); // fogadás indítása
                eArgs.RemoteEndPoint = _Socket.RemoteEndPoint;
                eArgs.SetBuffer(new Byte[20000], 0, 20000);
                eArgs.Completed += new EventHandler<SocketAsyncEventArgs>(Receive_Completed);
                _Socket.ReceiveAsync(eArgs);
            }
            else
            {
                if (ConnectionChanged != null)
                    ConnectionChanged(this, EventArgs.Empty);
            }
        }

        private void Receive_Completed(object sender, SocketAsyncEventArgs e)
        {
            if (e.SocketError == SocketError.Success && e.BytesTransferred > 0) // ha sikerült fogadni a tartalmat
            {
                Message message = Message.Deserialize(e.Buffer, e.BytesTransferred);

                switch (message.Code) // kódtól függően döltjük el, mit kell tenni
                { 
                    case MessageCode.Login:
                        _Player.PlayerId = Convert.ToInt32(message.Content); // azonosító visszakérése
                        break;
                    case MessageCode.GetOpenGames:
                        if (GamesReceived != null)
                            GamesReceived(this, new GamesEventArgs { Games = message.Content as Game[] });
                        break;
                    case MessageCode.CreateGame:
                        _Game = message.Content as Game;
                        break;
                    case MessageCode.JoinGame:
                        _Game = message.Content as Game;
                        if (GameStarted != null)
                            GameStarted(this, EventArgs.Empty);
                        break;
                    case MessageCode.JoinAccepted:
                        if (GameStarted != null)
                            GameStarted(this, EventArgs.Empty);
                        break;
                    case MessageCode.JoinRejected:                        
                        break;
                    case MessageCode.ChangeGameState:
                        if (GameStepReceived != null)
                            GameStepReceived(this, new GameStateEventArgs { PadPosition = Convert.ToInt32(message.Content) }); // játékállapot vissazkérése
                        break;
                    case MessageCode.EndGame:
                        if (GameOver != null)
                            GameOver(this, EventArgs.Empty);
                        break;
                }
                e.Dispose();

                SocketAsyncEventArgs eArgs = new SocketAsyncEventArgs(); // következő üzenet fogadása
                eArgs.RemoteEndPoint = _Socket.RemoteEndPoint;
                eArgs.SetBuffer(new Byte[20000], 0, 20000);
                eArgs.Completed += new EventHandler<SocketAsyncEventArgs>(Receive_Completed);
                _Socket.ReceiveAsync(eArgs);
            }
            else
            {
                _Socket.Close();
                if (ConnectionChanged != null)
                    ConnectionChanged(this, EventArgs.Empty);
            }
        }

        /// <summary>
        /// Új játék kezdése.
        /// </summary>
        public void NewGame()
        {
            SendMessage(MessageCode.CreateGame, new Game { FirstPlayer = _Player });
        }

        /// <summary>
        /// Nyitott játékok lekérdezése.
        /// </summary>
        public void GetOpenGames()
        {
            SendMessage(MessageCode.GetOpenGames);
        }

        /// <summary>
        /// Csatlakozás játékhoz.
        /// </summary>
        /// <param name="game">Játék.</param>
        public void JoinGame(Game game)
        {
            if (game != null)
            {
                SendMessage(MessageCode.JoinGame, game.GameId);
            }
        }

        /// <summary>
        /// Játék befejezése.
        /// </summary>
        public void EndGame()
        {
            SendMessage(MessageCode.EndGame, _Game);
        }

        /// <summary>
        /// Játék léptetése.
        /// </summary>
        /// <param name="movement">Ütőmozgás.</param>
        public void StepGame(Int32 movement)
        {
            SendMessage(MessageCode.ChangeGameState, movement);
        }

        /// <summary>
        /// Üzenet küldése.
        /// </summary>
        /// <param name="code">Üzenet kódja.</param>
        /// <param name="content">Üzenet tartalma.</param>
        private void SendMessage(MessageCode code, Object content = null)
        {
            if (_Socket != null && _Socket.Connected)
            {
                Byte[] buffer = Message.Serialize(code, content); // üzenet szérializációja

                if (buffer != null)
                {
                    SocketAsyncEventArgs sendAsyncEventArgs = new SocketAsyncEventArgs();
                    sendAsyncEventArgs.RemoteEndPoint = _Socket.RemoteEndPoint;
                    sendAsyncEventArgs.SetBuffer(buffer, 0, buffer.Length);
                    sendAsyncEventArgs.Completed += new EventHandler<SocketAsyncEventArgs>(Send_Completed);
                    _Socket.SendAsync(sendAsyncEventArgs);
                }
            }
        }

        private void Send_Completed(object sender, SocketAsyncEventArgs e)
        {
            if (e.SocketError != SocketError.Success || e.BytesTransferred == 0) // jelezzük, ha megszakadt a kapcsolat
            {
                _Socket.Close();
                if (ConnectionChanged != null)
                    ConnectionChanged(this, EventArgs.Empty);
            }
            e.Dispose();
        }
    }
}
