﻿using System;
using System.Net.Sockets;
using System.Threading;
using ELTE.Net.GameServer.Shared;

namespace ELTE.Net.GameServer.Console
{
    /// <summary>
    /// Játékkliens kezelő típusa.
    /// </summary>
    class ClientManager
    {
        private const Int32 BufferLength = 20000000; // puffer mérete
        private Socket _Socket; // hálózati csatoló

        /// <summary>
        /// Játékos lekérdezése.
        /// </summary>
        public Player Player { get; private set; }
        /// <summary>
        /// Jelenlegi játék lekérdezése.
        /// </summary>
        public Game CurrentGame { get; private set; }
        /// <summary>
        /// Kapcsolat fennállásának lekérdezése.
        /// </summary>
        public Boolean Connected { get { return _Socket.Connected; } }
                
        /// <summary>
        /// Játékkliens kezelő példányosítása.
        /// </summary>
        /// <param name="socket">Hálózati csatoló.</param>
        public ClientManager(Socket socket)
        {
            _Socket = socket;

            Thread receiveThread = new Thread(CommunicateWithClient); // új szál a kommunikációhoz.
            receiveThread.Start();
        }
        /// <summary>
        /// Kommunikáció végzése a klienssel.
        /// </summary>
        private void CommunicateWithClient()
        {
            try
            {
                while (_Socket.Connected)
                {
                    Byte[] buffer = new Byte[BufferLength];
                    Int32 receivedBytes = _Socket.Receive(buffer); // fogadás a puffertömbbe

                    Message message = Message.Deserialize(buffer, receivedBytes); // üzenet kiolvasása

                    switch (message.Code) // tevékenység végrehajtása a kód függvényében
                    {
                        case MessageCode.Login:
                            Player = (message.Content as Player);
                            Player.PlayerId = Guid.NewGuid().GetHashCode(); // egyedi értéket kap az azonosító
                            SendMessage(MessageCode.Login, Player.PlayerId);
                            break;
                        case MessageCode.GetOpenGames:
                            SendMessage(MessageCode.GetOpenGames, ServerManager.Instance.GetOpenGames(Player));
                            break;
                        case MessageCode.CreateGame:
                            CurrentGame = (message.Content as Game);
                            CurrentGame.FirstPlayer = Player;
                            ServerManager.Instance.CreateGame(CurrentGame); // létrehozzuk a játékot
                            SendMessage(MessageCode.CreateGame, CurrentGame); // visszaküldjük az azonosítót
                            break;
                        case MessageCode.JoinGame:
                            if (ServerManager.Instance.JoinGame(Convert.ToInt32(message.Content), Player)) // ha sikeres a játékhoz való csatlakozás
                            {
                                CurrentGame = ServerManager.Instance.GetGame(Convert.ToInt32(message.Content)); // lekérdezzük a játékot
                                CurrentGame.SecondPlayer = Player;
                                SendMessage(MessageCode.JoinAccepted, CurrentGame); // visszaküldjük a felhasználónak
                            }
                            else
                            {
                                SendMessage(MessageCode.JoinRejected, null); // különben nem küldönk semmit
                            }
                            break;
                        case MessageCode.ChangeGameState:
                            if (CurrentGame != null && CurrentGame.Phase == GamePhase.Playing) // ha a játék jelelleg folyamatban van
                            {
                                ServerManager.Instance.ChangeGameState(Player, CurrentGame, Convert.ToInt32(message.Content)); // akkor végrehajtjuk az állapotváltoztatást
                            }
                            break;
                        case MessageCode.EndGame:
                            if (CurrentGame != null && CurrentGame.Phase == GamePhase.Playing) // ha a játék jelelleg folyamatban van
                            {
                                CurrentGame.Winner = (message.Content as Game).Winner; // a győztest a klienstől kapjuk meg
                                ServerManager.Instance.EndGame(CurrentGame, Player); // befejezzük a játékot
                            }
                            CurrentGame = null; // majd töröljük a játékot
                            break;
                    }
                }
            }
            catch
            {
                Disconnect();
            }
        }
        /// <summary>
        /// Üzenet küldése a kliensnek.
        /// </summary>
        /// <param name="code">Üzenet kódja.</param>
        /// <param name="content">Üzenet tartalma.</param>
        public void SendMessage(MessageCode code, Object content = null) 
        {
            try
            {
                Byte[] buffer = Message.Serialize(code, content); // üzenet szérializálása
                if (buffer != null)
                    _Socket.Send(buffer); // majd küldése
            }
            catch
            {
                Disconnect();
            }
        }
        /// <summary>
        /// Kapcsolat bontása a klienssel.
        /// </summary>
        public void Disconnect()
        {
            if (CurrentGame != null && CurrentGame.Phase != GamePhase.Completed) // előbb bontjuk a nyitott játékokat
            {
                ServerManager.Instance.EndGame(CurrentGame, Player);
                // a másik játékos győzött
            }

            if (_Socket != null && _Socket.Connected)
            {
                _Socket.Shutdown(SocketShutdown.Both);
                _Socket.Close();
            }
        }
    }
}
